﻿using System;
using System.Windows.Forms;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.Samples.DirectX.UtilityToolkit;

using Direct3D = Microsoft.DirectX.Direct3D;

namespace kyuu
{

    // 1本の線
    public class 線
    {
        private VertexBuffer m_vb = null;	// 線

        private Matrix m_単位行列   = Matrix.Identity;

        // -----------------------------------------------------------------
        // マテリアル
        Direct3D.Material mtrl = new Direct3D.Material();
        System.Drawing.Color col;


        // アクセサ ----------------------------------------------------------------------
        // 単位行列
        public Matrix 単位行列
        {
            get { return m_単位行列; }
            set { m_単位行列 = value; }
        }
        /*
        Matrix M11 - M44
        11 12 13 14 -X
        21 22 23 24 -Y
        31 32 33 34 -Z
        41 42 43 44
        */


        public 線()
        {
            // 
            // TODO: コンストラクタ ロジックをここに追加してください。
            //
        }



        // ピッチ(x軸の回転)
        // d:回転角(ラジアン)
        public void RotateX_Pitch(float d)
        {
            Vector3 v = new Vector3(m_単位行列.M11, m_単位行列.M12, m_単位行列.M13);    // 回転の軸を表すベクトル
            Matrix m = Matrix.Identity;

            m.RotateAxis(v, d);         // 任意の軸を中心に行列を回転
            m_単位行列 *= m;            // 座標変換を合成
        }


        // ヨー(y軸の回転)
        public void RotateY_Yaw(float d)
        {
            Vector3 v = new Vector3(m_単位行列.M21, m_単位行列.M22, m_単位行列.M23);    // 回転の軸を表すベクトル
            Matrix m = Matrix.Identity;

            m.RotateAxis(v, d);         // 任意の軸を中心に行列を回転
            m_単位行列 *= m;            // 座標変換を合成
        }


        // ロール(z軸の回転)
        public void RotateZ_Roll(float d)
        {
            Vector3 v = new Vector3(m_単位行列.M31, m_単位行列.M32, m_単位行列.M33);    // 回転の軸を表すベクトル
            Matrix m = Matrix.Identity;

            m.RotateAxis(v, d);         // 任意の軸を中心に行列を回転
            m_単位行列 *= m;            // 座標変換を合成
        }






        public void CreateVB(   Device                  device,
                                System.Drawing.Color    カラー,
                                Vector3 開始,
                                Vector3 終了 )
        {
            this.col = カラー;

            // ----------------
            m_vb = new VertexBuffer(
                        typeof(CustomVertex.PositionColored),
                        2,
                        device,
                        Usage.WriteOnly,
                        CustomVertex.PositionColored.Format,
                        Pool.Default);


            // リソースのロック (CPUによる記憶領域へのアクセスを許可)
            CustomVertex.PositionColored[] verts = (CustomVertex.PositionColored[])m_vb.Lock(0, 0);

            int i = 0;
            verts[i].X = 開始.X;
            verts[i].Y = 開始.Y;
            verts[i].Z = 開始.Z;
            verts[i].Color = カラー.ToArgb();

            i++;
            verts[i].X = 終了.X;
            verts[i].Y = 終了.Y;
            verts[i].Z = 終了.Z;
            verts[i].Color = カラー.ToArgb();

            // リソースのアンロック
            m_vb.Unlock();



        }









        

        /// <summary>
        /// ボディに接続するパーツ表示
        /// </summary>
        /// <param name="device"></param>
        /// <param name="camera"></param>
        /// <param name="Body座標"></param>
        /// <param name="Body取付位置座標"></param>
        /// <param name="Body単位行列"></param>
        /// <param name="bPick"></param>
        public void Draw(
            Device              device,
            ModelViewerCamera   camera,
            Vector3             位置座標 )
        {
            Matrix m = new Matrix();

            m  = m_単位行列;                    // 回転
            m *= Matrix.Translation(位置座標);  // 移動

            device.Transform.World = m * camera.WorldMatrix;

            // -----------------------------------------------------------------
            // マテリアル
            mtrl.Diffuse = col;
            mtrl.Ambient = col;
            device.Material = mtrl;



            // -----------------------------------------------------------------
            // 線を描画
            // 頂点バッファをデバイスのデータ ストリームにバインド
            device.SetStreamSource(
                0,      // 0 から -1(ストリームの最大数)までの範囲のデータ ストリームを指定
                m_vb,	// 指定したデータ ストリームにバインドする頂点バッファ
                0);		// ストリーム先頭から頂点データ先頭までのオフセット (バイト単位)

            device.VertexFormat = CustomVertex.PositionColored.Format;

            // プリミティブのシーケンスをレンダリング
            device.DrawPrimitives(
                PrimitiveType.LineList,	// レンダリングするプリミティブの種類
                0,						// ロードする最初の頂点のインデックス
                1 );					// レンダリングするプリミティブ数

        }

        // バッファの開放
        public void Dispose()
        {
            if (null != this.m_vb)
            {
                this.m_vb.Dispose();
            }
        }


    }
}
