﻿using System;
using System.Windows.Forms;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.Samples.DirectX.UtilityToolkit;

using Direct3D = Microsoft.DirectX.Direct3D;

namespace kyuu
{
    public class 円柱
    {
        Mesh mesh = null;

        private Matrix m_単位行列   = Matrix.Identity;

        // -----------------------------------------------------------------
        // マテリアル
        Direct3D.Material mtrl = new Direct3D.Material();
        System.Drawing.Color col;


        // アクセサ ----------------------------------------------------------------------
        // 単位行列
        public Matrix 単位行列
        {
            get { return m_単位行列; }
            set { m_単位行列 = value; }
        }
        /*
        Matrix M11 - M44
        11 12 13 14 -X
        21 22 23 24 -Y
        31 32 33 34 -Z
        41 42 43 44
        */


        public 円柱()
        {
            // 
            // TODO: コンストラクタ ロジックをここに追加してください。
            //
        }



        // ピッチ(x軸の回転)
        // d:回転角(ラジアン)
        public void RotateX_Pitch(float d)
        {
            Vector3 v = new Vector3(m_単位行列.M11, m_単位行列.M12, m_単位行列.M13);    // 回転の軸を表すベクトル
            Matrix m = Matrix.Identity;

            m.RotateAxis(v, d);         // 任意の軸を中心に行列を回転
            m_単位行列 *= m;            // 座標変換を合成
        }


        // ヨー(y軸の回転)
        public void RotateY_Yaw(float d)
        {
            Vector3 v = new Vector3(m_単位行列.M21, m_単位行列.M22, m_単位行列.M23);    // 回転の軸を表すベクトル
            Matrix m = Matrix.Identity;

            m.RotateAxis(v, d);         // 任意の軸を中心に行列を回転
            m_単位行列 *= m;            // 座標変換を合成
        }


        // ロール(z軸の回転)
        public void RotateZ_Roll(float d)
        {
            Vector3 v = new Vector3(m_単位行列.M31, m_単位行列.M32, m_単位行列.M33);    // 回転の軸を表すベクトル
            Matrix m = Matrix.Identity;

            m.RotateAxis(v, d);         // 任意の軸を中心に行列を回転
            m_単位行列 *= m;            // 座標変換を合成
        }




        public void CreateVB(   Device device,
                                System.Drawing.Color カラー,
                                float 半径1,
                                float 半径2,
                                float 長さ)
        {
            this.col = カラー;

            // パーツ本体
            mesh = Mesh.Cylinder(device, 半径1, 半径2, 長さ, 6, 256 );   // 円柱
            /*
            円柱
            device  Microsoft.DirectX.Direct3D.Device。作成された円柱のメッシュに関連付けられているデバイスを表す Device オブジェクト。 
            radius1  System.Single。z 軸の負の側の面の半径。値は 0.0f 以上でなければなりません。(末端側) 
            radius2  System.Single。z 軸の正の側の面の半径。値は 0.0f 以上でなければなりません。(付根側)
            length  System.Single。z 軸方向の円柱の長さ。 
            slices  System.Int32。主軸を回転軸としたスライスの数。 
            stacks  System.Int32。主軸に沿ったスタックの数。 
             */

        }









        

        /// <summary>
        /// ボディに接続するパーツ表示
        /// </summary>
        /// <param name="device"></param>
        /// <param name="camera"></param>
        /// <param name="Body座標"></param>
        /// <param name="Body取付位置座標"></param>
        /// <param name="Body単位行列"></param>
        /// <param name="bPick"></param>
        public void Draw(
            Device              device,
            ModelViewerCamera   camera,
            Vector3             位置座標 )
        {
            Matrix m = new Matrix();

            m  = m_単位行列;                    // 回転
            m *= Matrix.Translation(位置座標);  // 移動

            device.Transform.World = m * camera.WorldMatrix;

            // -----------------------------------------------------------------
            // マテリアル
            mtrl.Diffuse = col;
            mtrl.Ambient = col;
            device.Material = mtrl;
            this.mesh.DrawSubset(0);   // 表示
        }

        // バッファの開放
        public void Dispose()
        {
            if (null != this.mesh)
            {
                this.mesh.Dispose();
            }

        }


    }
}
